/* Program09_2
 *
 * Ez a program az I2C1 buszon keresztül beolvassa a DS32231 RTC
 * 0 című regiszterét (másodpercek), s a legalsó bit értékét
 * kijelzi a PC13-ra kötött beépített LED-en.
 * A beépített LED lehúzásre világít (negatív logika).
 *
 * Kapcsolás:
 *   --------           -------------
 *  | STM32  |         | DS3231 RTC  |
 *  |    PB9 |---------| SDA     SQW |
 *  |    PB8 |---------| SCL         |
 *  |     5V |---------| VCC     VCC |
 *  |    GND |---------| GND         |
 *   --------           -------------
 * Felhasznált forrás: 
 * Mazidi et al., STM32 Arm Programming for Embedded Systems    
 * http://www.microdigitaled.com/ARM/STM_ARM_books.html
 *
 * Tested with Keil MDK-Lite v5.28 and STM32F1 DFP v2.3.0
 */
 
#include "stm32f10x.h"
#define SLAVE_ADDR 0x68    /* 1101 000.    DS3231 */

void I2C1_init(void);
int I2C1_byteRead(char saddr, char maddr, char* data);
void delayMs(int n);

int main(void) {
    char data;
  //--- PC13, a beépített LED konfigurálása --------
  RCC->APB2ENR |=  RCC_APB2ENR_IOPCEN;   // GPIOC órajel engedélyezés
  GPIOC->CRH &= ~(GPIO_CRH_CNF13|GPIO_CRH_MODE13); // pin13 CNF/Mode bitek törlése 
  GPIOC->CRH |= GPIO_CRH_MODE13_1;       // CNF:00, Mode:10 PPout, 2MHz

  I2C1_init();

  while (1) {
    I2C1_byteRead(SLAVE_ADDR, 0, &data);
    if (data & 1)
      GPIOC->ODR &= ~GPIO_ODR_ODR13;     // LED bekapcsolás
    else
      GPIOC->ODR |=  GPIO_ODR_ODR13;     // LED kikapcsolás

    delayMs(10);
  }
}

void I2C1_init(void) {
  RCC->APB1ENR |=  RCC_APB1ENR_I2C1EN;	      // I2C1 engedélyezése
  RCC->APB2ENR |= RCC_APB2ENR_AFIOEN;         // AFIO engedélyezése	
  RCC->APB2ENR |=  RCC_APB2ENR_IOPBEN;        // GPIOB engedélyezése
  AFIO->MAPR |= AFIO_MAPR_I2C1_REMAP;         // I2C1 REMAP=1 (SCL/PB8, SDA/PB9)
  GPIOB->CRH |= GPIO_CRH_CNF8|GPIO_CRH_MODE8; // PB8 ALT/OD kimenet (SCL)
  GPIOB->CRH |= GPIO_CRH_CNF9|GPIO_CRH_MODE9; // PB9 ALT/OD kimenet (SDA) 
  I2C1->CR1 =   I2C_CR1_SWRST;                // Szoftver RESET
  I2C1->CR1 &= ~I2C_CR1_SWRST;                // RESET vége
	/* FREQ = PCLK1/1_000_000         */ 
  I2C1->CR2 = 36;                             // PCLK1 = 36 MHz
	/* CCR = max(PCLK1/ClkSpeed/2,4)  */
  I2C1->CCR = 180;                            // Standard mód 100 kHz
	/* TRISE = PCLK1/1000000+1        */
  I2C1->TRISE = 37;                           // maximum felfutási idő
  I2C1->CR1 |=  I2C_CR1_PE;                   // I2C1 modul indítása
}

/* Ez a függvény beolvas egy adatbájtot (data) a saddr című 
 * I2C eszköz maddr című regiszteréből. Az egyszerűség kedvéért
 * nincs ellenőrzés, sem hibajelzés.
 */
int I2C1_byteRead(char saddr, char maddr, char* data) {
    volatile int tmp;

    while (I2C1->SR2 & 2);                  /* wait until bus not busy */
    
    I2C1->CR1 |=  0x100;                    /* generate start */
    while (!(I2C1->SR1 & 1));               /* wait until start flag is set */
    
    I2C1->DR = saddr << 1;                  /* transmit slave address + Write */
    while (!(I2C1->SR1 & 2));               /* wait until addr flag is set */
    tmp = I2C1->SR2;                        /* clear addr flag */
    
    while (!(I2C1->SR1 & 0x80));            /* wait until data register empty */
    I2C1->DR = maddr;                       /* send memory address */
    
    while (!(I2C1->SR1 & 0x80));            /* wait until data register empty */
    
    I2C1->CR1 |= 0x100;                     /* generate restart */
    while (!(I2C1->SR1 & 1));               /* wait until start flag is set */
    I2C1->DR = saddr << 1 | 1;              /* transmit slave address + Read */

    while (!(I2C1->SR1 & 2));               /* wait until addr flag is set */
    I2C1->CR1 &= ~0x400;                    /* Disable Acknowledge */
    tmp = I2C1->SR2;                        /* clear addr flag */

    I2C1->CR1 |= 0x200;                     /* generate stop after data received */

    while (!(I2C1->SR1 & 0x40));            /* Wait until RXNE flag is set */
    *data++ = I2C1->DR;                     /* Read data from DR */

    return 0;
}

/*----------------------------------------------------------------------------
    Késleltető eljárás a SysTick felhasználásával
 *----------------------------------------------------------------------------*/
void delayMs(int n) {
  int i;
  SysTick->LOAD = SystemCoreClock/1000-1;// Újratöltési érték 1 ms késleltetéshez
  SysTick->VAL  = 0;                     // Számláló törlése
  SysTick->CTRL = 0x5;                   // Engedélyezés, nincs interrupt, rendszer órajel
  for(i = 0; i < n; i++) {
    while((SysTick->CTRL & 0x10000)==0); // A COUNT jelzöre várunk
  }
  SysTick->CTRL = 0;                     // SysTick leállítása
}








